@@ -757,7 +757,7 @@ class Config:
         return self.actualBatchMode() and "keeptmp" not in self.optionMap
 
     def cleanWriteDirectory(self, suite):
-        if self.removePreviousThread and self.removePreviousThread.isAlive():
+        if self.removePreviousThread and self.removePreviousThread.is_alive():
             plugins.log.info("Waiting for removal of previous write directories to complete...")
             self.removePreviousThread.join()
             Config.removePreviousThread = None
@@ -1364,6 +1364,8 @@ class Config:
                              "default": 0.0}, "Which tolerance to apply when comparing floating point values in output")
         app.setConfigDefault("relative_float_tolerance", {
                              "default": 0.0}, "Which relative tolerance to apply when comparing floating point values")
+        app.setConfigDefault("floating_point_split", {
+                             "default": ''}, "Separator to split at when comparing floating point values")
 
         app.setConfigDefault("collate_file", self.getDefaultCollations(),
                              "Mapping of result file names to paths to collect them from")
